#if 0

N_("Bourbon Pork Loin")
N_("This is a modified version of a recipe found on a website.")
N_("1. Combine the spice rub ingredients, and rub all over the pork. Let sit at room temperature as you preheat your oven to 110ºC.")
N_("2. Place the pork on a skillet, then place in the oven and roast the pork for about 35 minutes. As the pork roasts, combine the glaze ingredients in a small saucepan; bring to a simmer over medium-low heat and simmer until dark and thickened, about 15 minutes. Set the glaze aside.")
N_("3. Remove the cooked pork and set on a cutting board to rest; loosely tent with tin foil. Increase the oven temperature to 260ºC. As the oven heats, place the skillet on the stove top, and add the oil, onion, and a few sprinkles of salt; cook until the onions are softened. Be careful because once onions start to caramelize they will quickly burn. Cover the pork liberally with glaze.")
N_("4. Roast a little bit more, less than 5 minutes. Let it rest and serve with smashed potatoes and vegs.")
N_("** If you decide to add apples, they go at the end of the 3rd step, before dripping glaze on the meat.")
N_("Some use maple syrup but I prefer the taste of pure honey mixed with the flavour of pork meat.")
N_("Same for apples; as some people add one or two. I don't like them but you if you fancy, feel free to add them.")
N_("Also, using lard makes this dish very heavy but using olive oil is pointless as its flavour will be missed. But if you have nothing else, use them. It won't hurt.")
N_("The Glaze")
N_("The Glaze")
N_("The Glaze")
N_("The Glaze")
N_("The Glaze")
N_("The Spice Cover")
N_("The Spice Cover")
N_("The Spice Cover")
N_("The Spice Cover")
N_("The Meat")
N_("The Meat")
N_("Others")
N_("Others")
N_("Others")
N_("Henry's original stew")
N_("My friend Heinrich made up this stew a winter Saturday.")
N_("It can be eaten with rice or alone.")
N_("Cook the meat with the species, not completely, just enough to get it toasted and mix the flavours. ")
N_("Add the vegetables and beans.  Cook a bit more.  ")
N_("Add abundant broth, cover the pot and let it cook on slow fire until is done. ")
N_("Can be served with rice  (like in the picture)  or alone.")
N_("Beans can be of any kind, and can be bought canned, in which case would be a can of each kind.")
N_("Juniper berries and Bay leaves are optional.")
N_(" ")
N_("Pesce alla sale")
N_("It's an easy but tasty way to prepare fish.")
N_("Despite all the salt, the result isn't salty but soft and nice.")
N_("Preheat the oven.")
N_("Stuff the fish with a mix of lemon and dill.")
N_("Combine the salt and eggs. Spread this mix over the fish.")
N_("Bake it 20' - 25'  or until the crust is brown. ")
N_("Serve it with plain rice.")
N_("Crispy olives")
N_("These olives in cheese dough are quick and easy to make and are great as a side dish or snack.")
N_("[image:1]Grate the cheese.")
N_("[image:4]Mix the flour and butter into a dough, add the cheese and salt and knead it until it is smooth.")
N_("[timer:00:30:00,Rise]Form the dough into a roll and let it rise in the fridge for 30 minutes.")
N_("[image:2]Cut the dough roll into 1 cm long slices and press the olives into them. You can roll the dough between your hands until it forms a little ball around the olive.")
N_("[timer:00:20:00,Bake]Bake for 20 minutes at [temperature:220C].")
N_("[image:0]Enjoy!")
N_("Neapolitan Pizza")
N_("Original pizza dough from Naples with tomato sauce on pizza stone.")
N_("Toppings are not included in the recipe.")
N_("The sauce can easily produced in bigger amounts and deep-freezed in freezer bags.")
N_("Dough: Dissolve the yeast in lukewarm water. Mix the flour, semolina and salt.")
N_("If brewer's yeast is not available, one can also use fresh yeast.")
N_("[timer:00:10:00]Put the yeast-water mixture to the flour and let it swell for 10 minutes.")
N_("[timer:00:15:00]Knead the dough with a kneading machine for at least 15 minutes. This is important so that the dough gets highly elastic.")
N_("[timer:08:00:00]Put the yeast dough covered for at least 8 hours (better 24 hours) in the refrigerator.")
N_("Sauce: Peel the shallots and the garlic and chop them finely.")
N_("Heat olive oil in a small pot and fry garlic and shallots shortly. Don't let them get brown.")
N_("[timer:00:30:00]Add the sieved tomatoes, tomato paste, red wine, oregano, salt and pepper. Let it boil slightly for 30 minutes so it gets that thick it does not run down the pizza.")
N_("[timer:01:30:00]Dough: Put the dough on a floured work surface. Create balls as big as fists (each 180-200g). Cover them with a moist kitchen towel and let them rise for an other 90 minutes at room temperature.")
N_("When you are ready to start, put the pizza stone in the oven and heat it up to [temperature:300C] (maximum temperature).")
N_("Take a dough ball and form with your hands into a round flat thin piece. Don't use a rolling pin, otherwise you destroy the yeast cultures.")
N_("Strew semolina on the shovel, so it can later easier slide of the shovel and put the dough piece on it.")
N_("Put the pizza sauce with a spoon on it, then pizza cheese (if you don't use e.g. mozzarella) and then the other toppings of your choice.")
N_("The pizza cheese should be below the toppings. This way the good ingredients are better presented.")
N_("If you use wet toppings like pineapples or mozzarella, dry them in a sieve before.")
N_("Don't put too much on the pizza.")
N_("Put the pizza with the shovel in the oven and bake it till it is brown. Duration is depending on oven temperature.")
N_("Pizza Dough")
N_("Pizza Dough")
N_("Pizza Dough")
N_("Pizza Dough")
N_("Pizza Dough")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Pizza Sauce")
N_("Quinoa Salad")
N_("Yummy salad that goes with just about anything.")
N_("Cook it like rice so bring quinoa and water to a boil  Reduce heat to medium-low, cover, and simmer until quinoa is tender and water has been absorbed, 10 to 15 minutes. Set aside to cool, this is important otherwise things get sticky ")
N_("Whisk olive oil, lime juice, cumin, 1 teaspoon salt, and red pepper flakes together in a bowl.")
N_("Combine everything but the cilantro, into a big bowl and toss to coat. Next season with salt & pepper and stir in the cilantro. Server right away or chill it for later.")
N_("If needed you can add some zesty Italian dressing for if it gets a little dry the next day.")
N_("Lemon Cake")
N_("My favorite Lemon Cake recipe. The cake is light and juicy.")
N_("Preheat the oven to [temperature:175C] - [temperature:195C].")
N_("First abrade the shell of the lemons. Squeeze the juice out of two (per serve) lemons.")
N_("Stir eggs and sugar foamy. Sieve the flour and add it with the vanilla sugar, baking powder, lemon shells and margarine one by one. Stir everything well.")
N_("[timer:00:20:00]Put the dough on a baking tray with baking paper and put it in the preheated oven. Bake it for 20min (middle rail).")
N_("Now mix the lemon juice little by little with the sieved powered sugar. Be sparing with the juice, the sugar icing must be pretty viscous.")
N_("As long as the cake is warm prick little holes with a fork allover the cake. Then spread the sugar icing on the warm cake. Let the cake cool down.")
N_("Henry's Goulash")
N_("Originally from Hungary, this dish is widespread in Central Europe.")
N_("This is my friend Henry's version of it.")
N_("Chop very minutely onions and garlic. Cut the meat in chunks. Chop red bell peppers in small squares.")
N_("Cook first the meat, alone without salt or spices.  Keep it warm in a separated bowl.")
N_("Fry in a mixture of butter and oil half of each pepper, onions and garlic.  When this is soft, add the tomato paste.")
N_("When everything is almost ready, add meat, wine, spices and bay leaves.  ")
N_("Put everything in the oven at a low temperature and cook it for some two hours.")
N_("Meantime the goulash is in the oven, chop the potatoes in sledges and cook them with salt & pepper.")
N_("Serve warm together.")
N_("If you don't have or don't want potatoes, you can use pasta instead.")
N_("Borscht")
N_("This is a traditional soup from East and Central Europe. There are an Ukrainian/Russian and a Polish version, but both a very similar so I put them together in a general recipe.")
N_("[image:0]Over an open flame or under a broiler, roast 1 carrot and half of the onion until charred, about 3 minutes.")
N_("[timer:00:08:00,Boil bones]Boil the bones in a large stockpot filled with water for 8 minutes, then drain and rinse thoroughly with cold water.")
N_("[timer:02:00:00,Simmer]Clean the stockpot, then return the bones to the pot along with the charred carrot and half onion and the peppercorns. Add enough cold water to cover the bones by at least 1″, about 4 quarts. Bring to a boil, then reduce the heat and simmer on medium-low until the meat easily separates from the bone, about 2 hours. Be sure to skim any foam or fat that accumulates on the surface. Add water to replace any that evaporates.")
N_("[timer:00:45:00,Bake beets]As the bones simmer, preheat the oven to [temperature:375F]. Place the beets in aluminum foil, drizzle with the olive oil, and wrap tightly. Roast until easily pierced with a fork, 45-60 minutes. Remove from the oven, let cool, then slip off the skins and cut the beets into French fry-sized strips. Set aside.")
N_("[image:0]When the bones are ready, strain the broth and discard the vegetables and peppercorns, then return the broth to the stockpot. Set the meaty bones aside to cool; once cool, pick apart the meat and set aside. Discard the bones or reserve for making stock.")
N_("[image:0]Coarsely chop the remaining half onion and grate the other carrot. In a skillet, warm the butter on medium heat, then add the chopped onion and sauté until softened, about 5 minutes. Add the minced garlic and tomato paste and sauté until aromatic, about 1 minute.")
N_("[timer:00:10:00,Cook]Add the sautéed onion and garlic paste to the broth, along with the beets, red wine vinegar, and grated carrot. Bring to a simmer on medium heat, then add the shredded cabbage and cubed potatoes. Return to a simmer and cook for 10 more minutes, skimming any excess fat from the surface, then stir in the meat, parsley, and lemon juice. Season with salt and pepper to taste, about 1/2 tsp. each.")
N_("[image:0]Cover, remove from the heat, and allow to brew for 20 minutes. Serve with a dollop of sour cream and fresh dill sprinkled on top. For a more flavorful soup, allow the soup to brew overnight and reheat the next day.")
N_("It's easy to convert this soup in vegan or vegetarian by not adding the meat and cream.")
N_("It will lose a bit of personality though.")
N_("If you're in a hurry, use a canned broil as base and add around 600 g of chopped meat without bones, and follow the rest of instruccions. This way it will be ready in less than an hour.")
N_("Ingredients may vary. In example lemon juice instead of vinegar and ground or powder pepper instead of the peppercorns.")
N_("Breakfast Oats Jar")
N_("This recipe is great for those who usually don't have time for breakfast. Prepare it in mason jars/portable food containers the night before, and you can have your homemade breakfast on-the-go! Make a couple batches in advance if you know you won't have time. This recipe is also very flexible as you can make it vegan, or gluten-free simply by substituting the components i.e. dairy milk with coconut milk, honey with agave, or use gluten-free oats. Also, any fruits of your choice, fresh or frozen.")
N_("[image:2]Add all the ingredients in a 16-oz container, or divide equally between two 8-oz containers.")
N_("[image:0]Mix well. Cover with lid(s), and place in the refrigerator for at least 4 hours or overnight. Thanks to the chia seeds, the mixture will develop a pudding-like texture.")
N_("[image:1]You can make ahead, but best to consume within 2-3 days.")
N_("Carbonara")
N_("\"Carbonara\" is a sauce that can be used with long pasta, like spaghetti or fettuccine.")
N_("Start by boiling about 1 litre of water for each 100g of pasta.")
N_("[image:1]")
N_("Once the water is boiling, pour 20g of coarse salt in it for every litre of water, and then throw the spaghetti in.")
N_("While the pasta is cooking, take a large bowl and put the parmesan cheese in it.")
N_("[image:2]")
N_("Add the whole egg and the yolks to the parmesan, and beat the eggs until you get a creamy sauce.")
N_("[image:3]")
N_("Put the guanciale in a small pan with a teaspoon of oil, and shallow fry it until the fat is transparent and the meat is crispy; then remove from the fire.")
N_("[image:5]")
N_("Once the pasta is ready, drain it and add it to the eggs and cheese. Then, add the guanciale.")
N_("[image:6]")
N_("Season with salt and pepper to your liking.")
N_("If you cannot find guanciale (cured pork cheek) you can substitute it with pancetta.")
N_("Classic Chicken Noodle Soup")
N_("A simple take on a typical American meal.")
N_("If you haven't butchered the chicken, do so. Depending on how meaty you want the soup, you can use more or less of the butchered parts (I recommend the dark leg meat for the soup). Use the remaining parts for another dish.")
N_("Wash the celery, carrots and herbs. Slice the carrots and celery. Coarsely chop the onion. Prepare the herbs using kitchen twine in a bouquet garni.")
N_("[timer:02:00:00]In a large pot, add the chicken, the stock, 2 carrots, 1 stalk of celery, the onion and the bouquet garni. Fill the pot with water to cover the chicken. Bring to a boil and simmer for at least 2 hours.")
N_("Using a cheese cloth strainer, strain the soup into another pot. Remove meat from the bones, gather any loose chicken and add it to the strained soup. Discard the bones and cooked vegetables.")
N_("Add the remaining carrots and celery to the soup. Bring to a boil and simmer for at least one hour. Season with salt and pepper to taste.")
N_("[timer:00:10:00]Add noodles and simmer for about 10 minutes, until the noodles are done.")
N_("Donauwellen")
N_("Donauwellen is a popular German cake with chocolate, vanilla cream and cherries. The name is due to the fact that the dough typically forms a wavy pattern.")
N_("[image:1]Mix the butter until it is creamy. Add sugar and egg.")
N_("[image:2]Mix the flour with the baking powder and mix it in. Separate a third of the dough and add the cocoa powder to it.")
N_("[image:4]Spread the light dough on a buttered baking sheet. Put the dark dough on top.")
N_("[image:4]Use a fork to make a wavy pattern.")
N_("[image:6]Spread the dried cherries on top of the dough.")
N_("[timer:00:30:00,Bake]Bake for 30 minutes at [temperature:200C].")
N_("[image:7]Use the milk, sugar and pudding to cook vanilla pudding and let it cool down while stirring it frequently. Slowly add spoonfuls of the butter, which should be at room temperature.")
N_("[image:8]Spread the butter cream on the cooled cake.")
N_("[image:9]Melt the chocolate frosting in a hot water bath and sprinkle it on top.")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Toppings")
N_("Toppings")
N_("Toppings")
N_("Toppings")
N_("Toppings")
N_("Gingered Carrot Soup")
N_("A nutritious vegan soup that can be made relatively quickly")
N_("Place carrots in large saucepan with water, cover, bring to a boil. Lower heat, simmer until very tender (10-15 minutes). While carrots are cooking, saute in olive oil all the onions, 5 minutes, then add garlic, ginger, salt, and spices. When all is well mingled, stir in lemon juice. Use food processor or blender to puree everything together (including all the toasted cashews). If you like a more textured soup, only half puree.")
N_("Garnish with parsley, chives, mint or scallions if you want. Top with cayenne pepper if you want a spicier soup. You can also add any leftover grains like rice, barley, couscous, etc. at the last minute.")
N_("This is my Aunt Wendy's recipe.")
N_("Grumbeerekiechle")
N_("Traditional potato hash from Alsace. Usually served with lettuce, sometimes with smoked salmon.")
N_("[image:1]Peel and grate the potatoes. Peel and slice the onions. Mix with the rest of the ingredients in a large bowl.")
N_("Put a frying pan on the hob at medium heat with a bit of oil.")
N_("[image:2]Form thin patties in the pan and brown them on both sides.")
N_("[image:0]Serve hot. Can be kept in a fridge and reheated in a micro-wave oven.")
N_("Hazelnuts Butter")
N_("Vegan Hazelnuts Butter that tastes like Nutella©, but better.")
N_("Make the Hazelnuts Milk (see my Hazelnuts Milk recipe)")
N_("Put the remaining paste in a bowl [image:0]")
N_("(With Chocolate) Mix melted 75% vegan chocolate")
N_("(With Cocoa Butter) Melt the cocoa butter [image:1]")
N_("(With Cocoa Butter) Mix the hazelnuts milk, melted cocoa butter and the cocoa powder until the mass gets uniform [image:2]")
N_("Enjoy [image:3]")
N_("Cocoa Butter Version")
N_("Cocoa Butter Version")
N_("Cocoa Butter Version")
N_("Base")
N_("Chocolate Version")
N_("Hazelnuts Milk")
N_("Simple Hazelnuts Milk with a smooth, naturally sweet flavor.")
N_("Preheat the over at [temperature:200C]")
N_("Place the hazelnuts in a tray [image:1]")
N_("Leave the tray in the oven for 15 minutes [timer:00:15:00]")
N_("Put water in the tray and peel the hazelnuts. Some will peel easily, while others will require manual peeling [image:2]")
N_("Leave the hazelnuts underwater at least for 4 hours, or overnight")
N_("Whisk the nuts and 1l of water with the blender until it gets thicker [image:3]")
N_("Filter the resulting liquid with a nut milk sack (voal sack) [image:4]")
N_("(Optional) You can make hazelnuts butter with the remaining paste (see my Hazelnuts Butter recipe) [image:5]")
N_("Enjoy your milk [image:6]")
N_("Kiwi-Mandarin Salsa")
N_("This salad is a sweet and sour addition which work well to most main courses, for example lasagna. The juicy fruits inside makes it a very refreshing salad.")
N_("[image:1]Wash the fruits and vegetables well.")
N_("[image:2]Peel, slice and cut the kiwi into triangular pieces.")
N_("[image:3]Peel the orange and cut it in pieces.")
N_("[image:4]Cut the red pepper in thin slices.")
N_("[image:0]Mix fruits and vegetables in a bowl.")
N_("[image:5]Add lime as you please.")
N_("[image:6]Enjoy!")
N_("Sylvia's Bratkartoffeln")
N_("Meatless version of a German classic.")
N_("Suitable for oven cooking.")
N_("Peel and chop potatoes to fry, preferably in thick chips.")
N_("Peel and chop the onions.")
N_("Prepare the cheese  (if it's not already).")
N_("Heat abundant oil and when is ready, carefully add the potatoes.")
N_("Once potatoes are half made, add the onions, and when everything is ready add the cheese.")
N_("** Alternatively, you can add the cheese at the moment of serving it.")
N_("** If you're using tofu, prepare the tofu first and add it at the moment of serving.")
N_("You can make it vegan, switching the cheese for vegan cheese or tofu.")
N_("You can use prepared cheese (like grated or shredded), but it tastes better if you use the fresh one and prepare it yourself.")
N_("Mantovana Cake")
N_("Mantovana cake (or Torta Mantovana in Italian) is a Tuscan classic. It is simple, yet rich and delicious. Very good for breakfast - Italian style - or as snacks, accompanied with coffee or tea. A small note; the vanilla baking powder is typical in Italy for baked sweet goods, and negates the need to add vanilla extract separately to a recipe. You are fine just using regular baking powder, but remember to add 1/2 teaspoon of vanilla extract. If you can't get hold of the vanilla powdered sugar for dusting, just use the regular powdered sugar.")
N_("[image:0]Preheat oven to [temperature:180C]. Line a 9-inch round cake pan with parchment paper and grease well.")
N_("[image:0]Combine eggs and sugar in a mixing bowl. Beat (using a hand mixer) until light and fluffy.")
N_("[image:0]Into the mixture, add half of the flour and give it a little mix. Add the rest of the flour, together with the baking powder and lemon zest. Mix until the batter is just combined. Don't overmix. ")
N_("[image:0]Add the melted butter to the mixture. Using a spatula, stir to mix well.")
N_("[image:0]Pour the batter into a 9-inch round baking pan. Sprinkle the almonds on top making sure to cover the surface evenly.")
N_("[timer:00:25:00,Bake]Bake for 25-30 minutes or until a cake tester inserted into the center cake comes out clean. The cake should have a nice golden colour.")
N_("[image:0]Let rest in the pan for 10 minutes, then invert onto a cooling rack to cool completely.")
N_("[image:0]Before serving, dust the top generously with vanilla powdered sugar.")
N_("Nasi Lemak (Coconut Rice)")
N_("This is a truly Malaysian dish, very typically eaten for breakfast. It is a delightful combination of boiled rice, coconut milk and pandan (screwpine leaves), eaten with the sambal (hot sauce), and accompaniments such as sliced cucumbers, hard-boiled eggs, fried dried anchovies and fried groundnuts.")
N_("See my recipe for Sambal Ikan Bilis (Hot Sauce with Anchovies) to complete this dish.")
N_("In this recipe, I outlined the steps to cook the rice on a stove. If you have a rice cooker, then you'll only have two steps; 1) Combine all ingredients in the pot of the rice cooker, and 2) Hit the \"cook\" button.")
N_("Prepare the rice; rinse, soak for 30 minutes, and drain. You can also use any long-grain rice instead of basmati.")
N_("[image:0]Peel the shallots, garlic and ginger, then slice them thinly.")
N_("[image:1]Mix the coconut milk, water, shallots, garlic and ginger in a medium-sized pot. Tie the pandan leaves in a knot, and add it to the mixture. If you can't get a hold of pandan, use one stalk of lemongrass as a substitute. Peel the tough outer leaves, slice off the lower bulb and the upper end of the stalk that is green and woody. Crush the lower bulby end, and throw it into the mixture. Season the mixture with salt. It should taste salty enough, and a bit more.")
N_("Bring the mixture to a boil.")
N_("When the mixture has come to a boil, stir in the salt and fenugreek seeds (if using), and bring it back to a gentle simmer.")
N_("Cover the pot and turn the heat down to low. Don't take off the lid while the rice is cooking.")
N_("Start checking the rice at around 20 minutes. When done, the rice will be firm but tender, and no longer crunchy.")
N_("Turn off the heat and let stand about 5-7 minutes, covered.")
N_("Remove the lid, and fluff up the rice with a fork. Let it stand for a few more minutes to dry out the steam.")
N_("[image:2]Serve with sambal ikan bilis (hot sauce with anchovies), sunny-side up or hard-boiled egg, sliced cucumbers.")
N_("Non-knead buns")
N_("Very quick fresh bread for breakfast with 5 minutes of work.")
N_("[image:1]Preparation: Mix together all of the ingredients in a large bowl into a thick porridge-like dough. Do not knead the dough, just mix it together. ")
N_("Cover and leave to rise in room temperature for 8-12 hours. This can be done for example the previous night so you can bake them in the morning.")
N_("Baking: Heat your oven to [temperature:250C].")
N_("[image:3]The dough will be more than double in size and full of bubbles and very sticky. Pour a generous amount of flour on your baking surface so it doesn't stick, and pour the dough over it.")
N_("[image:4]Form the dough into a thick snake, about the diameter of your arm and cut bun-sized pieces with scissors onto the baking sheet.")
N_("[timer:00:15:00]Bake until nice and brown, this will take approximately 15 minutes.")
N_("[image:5]You can substitute half of the wheat flour with oats, seeds or different kind of flour if you want variety.")
N_("[image:0]Enjoy!")
N_("Pan pizza")
N_("Pizza is an all-time favorite.")
N_("Put the flour, yeast, water and olive oil in a bowl and stir it up with your hands or a spoon until there's no dry flour left, and let it be.")
N_("Cover the bowl tightly in plastic wrap and let it sit at room temperature overnight. ")
N_("The next day it should have rosen quite a bit. Dump it all out onto a floured surface, then divide it into balls.")
N_("Spread olive oil in a 10-inch cast iron skillet or round cake pan and place a dough ball in the middle.")
N_("Rub the dough ball around and flip it a couple times until it is completely coated in oil and the oil completely coats the bottom and sides of the pan. Cover in plastic, and let it sit for another couple of hours without touching it.")
N_("The dough should have spread on it own (takes about 2 hours). Lift the sides of the dough to let big air bubbles out.")
N_("Put in the oven at [temperature:280C]. (or as hot as your oven goes) for about 15 minutes. Peek underneath and if your pizza is a little pale, you can finish on the stovetop directly over medium heat until it is as dark and crisp as you like it.")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Toppings")
N_("Toppings")
N_("Toppings")
N_("Pineapple Upside Down Cake with Hot Buttered Rum Sauce")
N_("This is a delicious, boozy update on a classic cake. The combination of rum and pineapple gives it a distinctively Caribbean flavor.")
N_("Preheat the oven to [temperature:175C] (gas mark 4).")
N_("Grease a round 20 cm cake tin.")
N_("Line the bottom of the tin with as many whole pineapple rings as will fit. Cut the remaining rings in half and use them to line the sides of the tin.")
N_("Crack the eggs into a bowl and add the caster sugar. Whisk until pale and light (an electric whisk is good for this, but not essential).")
N_("In a different bowl, combine the flour and baking powder, and then sift into the egg and sugar mixture, mixing as you go.")
N_("Melt the butter and combine with the milk, rum and vanilla extract. Zest half the lemon and add.")
N_("Mix the wet mixture into the rest of the ingredients.")
N_("[timer:00:40:00,Bake]Poor the cake mixture into the cake tin and bake for around 40 minutes, until the top is light brown and an inserted skewer comes out clean.")
N_("[timer:00:05:00,Cool]Leave the cake to cool for 5 minutes, then turn out on to a rack to cool completely.")
N_("When it's time to eat, make the sauce: combine the muscovado sugar and cream in a saucepan and heat until the sugar has dissolved. Add the rum and simmer until it has thickened a little, then add the butter and let it melt into the sauce.")
N_("Pour the hot sauce over the cake and serve.")
N_("Sauce")
N_("Sauce")
N_("Sauce")
N_("Sauce")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Cake")
N_("Pizzaburger")
N_("Tomato dough with hamburger and cheese topping")
N_("Dissolve yeast in warm water. Let stand for 2 minutes.")
N_("Add 1/2 cup tomato sauce, oil, salt, 1 tsp chili powder, and sugar. Mix well.")
N_("Gradually stir in flour. Dough will become sticky.")
N_("Optional: Let rise covered in a warm place for about an hour.")
N_("Brown diced onion and hamburger in frying pan.")
N_("Add remaining 1 1/2 cups tomato sauce, salt, pepper, and 2 tsp chili powder.")
N_("Remove from heat.")
N_("Spread dough on greased cookie sheet (will be somewhat springy and pull back as it is spread out).")
N_("Top with hamburger mixture and cover with cheese slices.")
N_("[timer:00:20:00]Bake at [temperature:400F] for 20 to 30 minutes until dough is baked.")
N_("Råkost")
N_("A sweet and sour addition to your dinner. This salad has many variations of it and is a common winter side dish in Denmark.")
N_("[image:1]Grate the carrots on the fine side of the grater.")
N_("[image:0]Spread the carrot equally in a platter.")
N_("[image:0]Divide and cut the apples into small bite-size pieces.")
N_("[image:0]Distribute the apples and blueberrys on top of the carrot.")
N_("[image:0]Divide a lime in two and squeeze its juices across the whole platter.")
N_("[image:2]Sprinkle a bit with linseeds if you want and it's then ready to eat. :-)")
N_("Roots Pie")
N_("This vegetable pie is my own spin off a traditional recipe. It has great taste, crusty dough and fancy colors too.")
N_("[image:12]Turn on the oven at [temperature:200C].")
N_("[image:2]Pour flour on the table, add salt. Divide the butter into small pieces and add water.")
N_("[image:3]Use the butter, water and yoghurt to work it all into a dough.")
N_("[image:4]Find a pie form.")
N_("[image:5]Takes pieces of the dough and start filling the pie form, until the whole form is full.")
N_("[image:6]Find a fork and punch holes in the bottom.")
N_("[timer:00:15:00,Pre-bake]Pre-bake the dough for 15 minutes in the oven.")
N_("[image:7]Peel and chop the vegetables. Beetroots color a lot, so be careful. Do them last.")
N_("[image:8]Mix eggs, cheese, yoghurt, salt and pepper in a bowl.")
N_("[image:9]Add beetroot, kale and lastly carrot to the pie.")
N_("[image:10]Pour the contents of the bowl over the vegetables so all of the pie is covered.")
N_("[image:11]Decorate with extra cheese and linseeds for extra sazz.")
N_("[timer:00:35:00,Bake]Bake the pie in the oven for 35 minutes.")
N_("[image:1]Enjoy!")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Dough")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Contents")
N_("Rosemary Scalloped Potatoes")
N_("A holiday twist on a classic American comfort food dish. Easy to make and sure to please.")
N_("Substitutions and conversions:")
N_("5 potatoes = 5 cans sliced potatoes")
N_("1 stick butter = 8 Tbsp butter")
N_("4 cloves garlic = 1/2 tsp powdered garlic")
N_("8 oz cream cheese = 1 pouch")
N_("1 cup half and half = one small carton")
N_("1 cup grated parmesan = 1 small bag")
N_("Preheat oven to [temperature:350F].")
N_("Butter a 13x9 baking dish.")
N_("Into buttered baking dish:")
N_("- Add 5 thinly sliced (use a mandoline) russet potatoes, or 5 cans rinsed pre-sliced potatoes.")
N_("- Salt and pepper potatoes (more will be added to creamy mixture later), stir lightly to separate slices.")
N_("In a frying pan on medium to medium-low heat:")
N_("- Melt 1 stick of butter.")
N_("- Add 1 chopped onion, garlic. Saute until translucent (about 5 minutes).")
N_("- Add 8 oz cream cheese (quartered to help it melt faster).")
N_("- When cream cheese is melted, add 1 cup half and half.")
N_("- Add salt and pepper to taste. (Don't be afraid to use more salt than you expect — it's a large dish, and potatoes need flavoring.) Recommend 1/2 Tbsp salt and 1/2 tsp pepper.")
N_("- Add finely chopped fresh rosemary and chopped green onions.")
N_("- Add 1/2 cup grated parmesan and melt into mixture.")
N_("To bake:")
N_("- Pour mixture over potatoes, stirring a bit to help the sauce cover everywhere. If using canned potatoes, be sure to separate the slices which are otherwise stuck together and prevent the mixture from seeping in.")
N_("- Spread remaining cheese over top to provide a nice texture and cheese flavoring.")
N_("- Bake for 1 to 1 1/2 hours, until liquid is bubbling and top cheese is browning.")
N_("Note: Don't be afraid to use extra parmesan cheese and cream (half and half). This recipe makes just enough sauce, but extra creaminess never hurts, especially if you're using canned sliced potatoes. Garlic can be increased to taste, but be careful not to let the garlic overtake the rosemary.")
N_("Salmon Fried Rice")
N_("A simple dish to throw together especially when you have leftover cooked rice and leftover baked/grilled salmon. ")
N_("What if you want to make Salmon Fried Rice now and don't have leftover rice and salmon? Easy; cook the rice and let it cool completely (uncovered for 15 minutes immediately after rice is just cooked, and then cover until you're ready to cook). Use frozen salmon fillets, but make sure to thaw sufficiently.")
N_("In this recipe, I'll show you how to do it.")
N_("[image:0]Preheat the oven to [temperature:200C] and line a baking dish with aluminium foil. Rinse the salmon and pat dry. Drizzle a bit of olive oil and sprinkle salt on both sides of salmon. Bake in the oven for 15-20 minutes, until the flesh is cooked through. Alternatively, you can grill or pan-fry the salmon.")
N_("[image:1]Allow the salmon to cool, flake the flesh and set aside.")
N_("[image:2]Peel and crush the garlic. Finely chop the spring onions.")
N_("[image:3]Heat the sesame oil in a wok, or skillet, and when hot, add the spring onions and garlic. Toss for about 1 minute.")
N_("[image:4]Add the chili powder and toss for about 1 minute.")
N_("[image:5]Add a couple tablespoons of water to loosen up the mixture, then stir to mix.")
N_("[image:6]Add the rice and break the chunk of rice so the rice fry out and mix well together.")
N_("[image:7]Add the fish and toss the rice again.")
N_("Add the fish sauce to taste. Fish sauce has a strong smell; if you don't fancy it, you can substitute it with soy sauce. Mix well to incorporate all the flavours.")
N_("[image:8]Serve immediately with sunny-side up eggs, and garnish with spring onions.")
N_("Sambal Ikan Bilis (Hot Sauce with Dried Anchovies)")
N_("The sambal (hot sauce) is an essential component of nasi lemak. For this sambal, dried anchovies (ikan bilis) are used. These little salted fish can be found in Asian specialty supermarkets.")
N_("[image:0]Prep the dried chillies; remove the stalks, cut in 1-inch pieces and remove the seeds. Rinse, then soak in hot water to soften.")
N_("[image:1]Prep the dried anchovies; rinse, and soak in hot water to soften. Then drain and pat dry.")
N_("Peel the shallots, ginger and garlic. Put them in a blender together with the prepped dried chillies. Add 1 cup of water. Blend to a puree.")
N_("[image:2]Heat 1/4 cup of coconut oil in a pan and stir-fry the chilli-spice puree until the oil separates, about 20 minutes.")
N_("[image:3]Pan-fry the dried anchovies in a separate small pan/skillet - using enough oil to cover bottom of pan - until light golden brown. Set aside.")
N_("Take the large onion, quarter it and slice across. Add these onions and the fried anchovies into the chilli-spice mixture and cook until the onion is transparent.")
N_("Add the coconut milk, tamarind paste and remaining 1 cup water, and bring to a boil on medium heat.")
N_("[image:4]Turn the heat down to low. Season with sugar and salt. The sambal must have a hint of sweetness.")
N_("[image:5]Let the sambal simmer uncovered until oil appears on the surface. Stir every 15 minutes. This will take about an hour. The sambal will turn into a deep red shade.")
N_("[image:6]Serve with nasi lemak (coconut rice).")
N_("Savoury cakes")
N_("These savory cakes can be served as appetizers or as main course. There are several variations that you can bake with the same base depending on your mood and tastes. Quite common at French parties.")
N_("Prepare the batter: melt butter in a large bowl, then mix with the eggs, flour and yeast in a bowl.")
N_("Add the ingredients matching the variation you chose, preferably sliced or diced to turn them into small bits, and mix again. Try coming up with your own variations!")
N_("Pour the batter in a loaf tin.")
N_("[timer:00:40:00,Bake]Put in the oven at [temperature:180C] for 40 minutes.")
N_("[image:1]Serve cold, cut in slices or cubes that people will get with a toothpick.")
N_("Cake aux lardons")
N_("Cake aux lardons")
N_("Cake aux lardons")
N_("Cake aux lardons")
N_("Batter")
N_("Batter")
N_("Batter")
N_("Batter")
N_("Cake au thon et à la tomate")
N_("Cake au thon et à la tomate")
N_("Cake chèvre-épinards")
N_("Cake chèvre-épinards")
N_("Cake noix-chorizo")
N_("Cake noix-chorizo")
N_("Cake noix-chorizo")
N_("Cake jambon-champignons")
N_("Cake jambon-champignons")
N_("Cake jambon-champignons")
N_("Spanish Omelet")
N_("Traditional Spanish Omelet, also known as \"Tortilla de Patata\"")
N_("[image:0]Put a bit of oil in a pan and cover the surface with sliced onions.")
N_("[image:3]Meanwhile, peel the potatoes and cut them in small pieces.")
N_("[image:2]Once the onions start to take colour, put the potatoes into the pan and add some salt.")
N_("[image:1]Cover the pan and stir the contents from time to time, until potatoes are well cooked.")
N_("[image:7]Put the eggs in a big bowl and add some salt")
N_("[image:6]Whip the eggs.")
N_("[image:5]Put the contents of the pan in the bowl and mix it well.")
N_("[image:4]Add some oil to the pan and put the whole mix back into the pan.")
N_("[image:8]Once the lower side is cooked, with the help of a plate, turn the omelet.")
N_("Repeat the process until it is well cooked.")
N_("Tagliatelle alla Romana")
N_("Pasta is one of my favourite things in the world, and this is one of my favourity pasta dishes. It's gutsy Italian food at its best.")
N_("It's important to have good quality chicken livers for this recipe - get free range organic if possible. You can use dried pasta if you're unable to get fresh.")
N_("Rinse the chicken livers and cut out any out any bits of sinew. Dry and season with salt and pepper.")
N_("Finely chop the red onion.")
N_("Melt half the butter in a frying pan, then add the olive oil and cook the onion until soft, without letting it brown.")
N_("Add the livers to the pan and fry until lightly browned. Then add the tomatoes and cook on a low heat for 20 minutes, stirring occasionally.")
N_("Put a deep pan of water on to boil and add a good pinch of salt to the water.")
N_("Remove the livers from the pan and chop finely. Then add back to the sauce and continue to cook. Check for seasoning and add more salt and pepper if necessary.")
N_("Cook the pasta for 3 minutes if fresh (or put on earlier and follow packet instructions if using dry pasta).")
N_("Drain the pasta, retaining a cup of the pasta water. Add the pasta to the sauce along with the remaining butter. Mix and cook on a low heat for several minutes, to allow the pasta to absorb the sauce.")
N_("If the sauce needs loosening, add a bit of the pasta cooking water.")
N_("Serve with plenty of black pepper.")
N_("Tofu ‘Egg’ Scramble")
N_("This vegan breakfast staple is perfect on a lazy saturday morning.")
N_("[image:0]Chop the onion. Open up the tofu package and slice it up, then crumble with your hands.")
N_("[timer:00:03:00,Sautee]Sautee the onion, tofu, and peppers for 3-5 minutes using the vegan butter.")
N_("[timer:00:05:00,Cook]Turn heat down to medium and add the rest of the ingredients – cook for 5-7 minutes more.")
N_("Traditional South Indian Tomato Chicken Curry, Fast Version")
N_("A quick and easy version of a traditional South Indian chicken curry dish, learned from a Mallu immigrant and cook.")
N_("Preparation:")
N_("Cube 5 large chicken breasts (about 4 cups) into 1-1.5 inch cubes")
N_("Thinly slice 1 large onion, chopping the final quarter of the onion")
N_("Chop 2 medium tomatoes into large chunks")
N_("Cooking:")
N_("In a large pot, heat 1 tbsp olive oil at medium/medium-high heat. Once oil is hot, add mustard seeds and fry until they pop (cover the pot to prevent splashing). Too much heat will burn the seeds.")
N_("Once seeds have popped, turn down the heat to medium and add thinly sliced onion. Cover to retain moisture. Cook until translucent, stirring frequently. Add small amounts of water as necessary.")
N_("While onions are cooking, blend remaining chopped onion and tomatoes to create a puree. Add water if necessary.")
N_("Once onions are translucent, increase heat to medium-high and immediately add all spices. Toast for 30 seconds until aromatic.")
N_("Add the rest of the olive oil (2 more tbsp) and tomato and onion puree from blender. Cook until raw smell of onions and tomatoes is going — up to 5 minutes.")
N_("Turn down heat to medium. Add chopped chicken breast, cover, and allow to slowly cook. Add small amounts of water as necessary to achieve desired sauce consistency. (Chicken will release its own fat and water into the sauce, so wait a few minutes before adding extra.)")
N_("Can be served immediately or left to marinate in curry sauce, then reheated. Serve with rice or chapati/heated flour tortilla.")
N_("To make a more full-flavored curry, try adding garam masala (ground cardamom, cloves, fennel seeds, and nutmeg), along with fresh curry leaves.")
N_("White fish for people who dont like fish.")
N_("It is a fantastic dish for those of us that dont love fish, but know we have to eat it to stay healthy.")
N_("The key to get this dish good is to have high temperature on the stove when first frying the fish. Then turn it down after the initial fry.")
N_("Set the stove to high (8/10), then put a good amount of oil in it. Any type of oil will do.")
N_("Remove bone and skin from the fish (if any), and add salt and pepper. Give the stove some minutes to get really hot. Then put the fish in. 2-3 min on each side.")
N_("Turn the temperature down to low (3/10). Add two spoons of butter, some thyme, rosemary and garlic. When the butter have melted, use a spoon to poor the melted butter and spice mix over the fish fillet.")
N_("After about 5 minutes your fish will be done. You can serve it with potatoes, rice, or noodles. Everything works.")
N_("Wholesome Crackers")
N_("Crackers are a fulfilling alternative snack and keeps your stomach going. Crackers consist of seeds, oat and flour. The recipe described here is very flexible. You can use whichever seeds, nuts and flour types you may have. I've personally tried almonds, sesame seeds, chia seeds, rye grains, millet and  buckwheat.")
N_("Turn on the oven at [temperature:200C]. Mix all dry ingredients (seeds, flour, oatmeal, baking powder and salt) together. Mix oil. Then add water gradually while mixing until the dough is firm and pasty. If you add too much water, the dough will end up sticking too much to the baking paper.")
N_("Cut a sheet of baking paper at the size of a baking tray and place the dough on it. Cut another sheet of baking paper and put on top. Use a rolling pin on top of the second baking sheet to flatten the dough. Make the dough as thing as possible. Afterwards, take off the second sheet. Bring forth a knife and cut creases from horizontally and vertically to form squares (see example on the picture). Optionally decorate with extra seeds on top before inserting into the preheated oven.")
N_("Wait for the crackers to become crisp, this takes around 17 minutes. You can check if they are done by touching the crackers or by pulling up a bit in the baking sheet and see if the crackers in the middle still bends.  The crackers take on a yellow golden color when they are done.")
N_("Take out the crackers and leave them on the kitchen table to cool down for a few minutes.")


#endif
