label.big.cuisine.heading,
label.cuisine.heading {
  color: white;
  margin: 32px 32px 0;
  font-weight: normal;
  font-size: 18pt;
}

label.big.cuisine,
label.cuisine {
  border-width: 0;
  color: white;
  margin: 0 32px 32px;
  font-size: 11pt;

}

/* Can be overridden for individual tiles like this:
label.cuisine.heading.mediterranean,
label.cuisine.mediterranean {
  color: black;
}
*/

.cuisine.view.tile {
  background-repeat: no-repeat;
  background-size: 100%;
  background-position: 50%;
  border-width: 0;
  min-width: 396px; /* FIXME: avoid smaller columns */
}


.cuisine.view.tile.american {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/american.png');
}
.cuisine.view.tile.chinese {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/chinese.png');
}
.cuisine.view.tile.indian {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
  url('file://@pkgdatadir@/images/indian.jpg');
}
.cuisine.view.tile.italian {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/italian.png');
}
.cuisine.view.tile.french {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/french.jpg');
}
.cuisine.view.tile.greek {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/greek.png');
}
.cuisine.view.tile.mexican {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/mexican.png');
}
.cuisine.view.tile.mediterranean {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/mediterranean.png');
}
.cuisine.view.tile.nordic {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/nordic.jpg');
}
.cuisine.view.tile.turkish {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.8)),
              url('file://@pkgdatadir@/images/turkish.png');
}

.cuisine.big.view.tile.american {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-american.png');
}
.cuisine.big.view.tile.chinese {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-chinese.png');
}
.cuisine.big.view.tile.indian {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-indian.jpg');
}
.cuisine.big.view.tile.italian {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-italian.png');
}
.cuisine.big.view.tile.french {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-french.jpg');
}
.cuisine.big.view.tile.greek {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-greek.png');
}
.cuisine.big.view.tile.mexican {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-mexican.png');
}
.cuisine.big.view.tile.mediterranean {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-mediterranean.png');
}
.cuisine.big.view.tile.nordic {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-nordic.jpg');
}
.cuisine.big.view.tile.turkish {
  background: linear-gradient(to bottom, transparent, rgba(0,0,0,0.5)),
              url('file://@pkgdatadir@/images/big-turkish.png');
}
