@import url("resource:///org/gnome/Recipes/recipes.css");

.note {
  background: #8091c4;
}

.note label,
.note image {
  color: white;
  margin: 5px 0 5px 10px;
}

.note.content {
  background: #a8c3e1;
  padding: 10px;
}

.note:backdrop {
  background: #b1b6c4;
}

.note.content:backdrop {
  background: #cbd6e1;
}

row.drag-icon {
  background: white;
  border: 1px solid alpha(black, 0.3);
  outline-width: 0;
}
